//-----------------------------------------------------------------------------
//                                  CNeuron                                   !
//-----------------------------------------------------------------------------

class CNeuron {

protected:

  CXmlElement *FXmlElement;
  CNet        *FNet;
  CLink       *FRightLinks[];

public:
  
  // Constructor procedures 
  
  void          CNeuron ();
  virtual void ~CNeuron ();
  
  virtual bool Init (CXmlElement *aXmlElement, CNet *aNet);
  virtual void Upload ();
//  virtual void Clear ();
  
  // Main procedures 

  virtual bool Run ();
  virtual int GetLayer ();
  virtual float GetValue ();
  virtual float GetdFdLink (const CLink *aLink);
  virtual float GetdFdNeuron (const CNeuron *aNeuron);
  
  // Service procedures

  CXmlElement* GetXmlElement (); 
  CNet*        GetNet (); 
  
  int    GetRightLinkCount (); 
  CLink* GetRightLink (int idx);
  int    GetRightLinkIdx (CLink* aLink);
  int    AddRightLink (CLink* aLink);
  void   DelRightLink (int idx); 
  void   DelRightLink (CLink* aLink); 
};

//--------------------------------------------------------------------------------/
//                              CNeuron :: implementation                         /
//--------------------------------------------------------------------------------/
  
void CNeuron::CNeuron () 
{
  FXmlElement=NULL;
  FNet = NULL;
  ArrayResize(FRightLinks,0);
};

void CNeuron::~CNeuron () 
{
};

bool CNeuron::Init (CXmlElement *aXmlElement, CNet *aNet) 
{
  FNet = aNet;
  FXmlElement=aXmlElement;
  return (FXmlElement.GetAttribute("NAME")!=NULL);
};
  
void CNeuron::Upload () 
{
};

/*void CNeuron::Clear () 
{
  FNet=NULL;
  FXmlElement=NULL;
  ArrayResize(FRightLinks,0);
};*/
  
bool CNeuron::Run() 
{
  return false;
};
  
int CNeuron::GetLayer ()
{
  return INT_MAX;  
};

float CNeuron::GetValue () 
{
  return FLT_MAX;
};

float CNeuron::GetdFdLink (const CLink *aLink) 
{
  return 0;
};
  
float CNeuron::GetdFdNeuron (const CNeuron *aNeuron) 
{
  if (aNeuron == GetPointer(this))
    return 1.0f;
  else 
    return 0.0f;
};
  
CXmlElement* CNeuron::GetXmlElement () 
{
  return FXmlElement;
};
  
CNet* CNeuron::GetNet () 
{
  return FNet;
};

int CNeuron::GetRightLinkCount () 
{
  return (ArraySize(FRightLinks));
};

CLink* CNeuron::GetRightLink(int idx) 
{
  return (FRightLinks[idx]);
};

int CNeuron::GetRightLinkIdx (CLink* aLink)
{
  for (int i=ArraySize(FRightLinks)-1; i>=0; --i)
    if (FRightLinks[i]==aLink)
      return i;
  return -1;
};

int CNeuron::AddRightLink(CLink* aLink) 
{
  ArrayResize(FRightLinks,ArraySize(FRightLinks)+1);
  FRightLinks[ArraySize(FRightLinks)-1]=aLink;
  return (ArraySize(FRightLinks)-1);
};
  
void CNeuron::DelRightLink(int idx) 
{
  for (int i=idx+1; i<ArraySize(FRightLinks); ++i)
    FRightLinks[i-1]=FRightLinks[i];
  ArrayResize(FRightLinks, ArraySize(FRightLinks)-1);
};  

void CNeuron::DelRightLink (CLink* aLink)
{
  DelRightLink(GetRightLinkIdx(aLink));
};  

