//-----------------------------------------------------------------------------
//                                  CNeuronIndicator                          !
//-----------------------------------------------------------------------------

class CNeuronIndicator : public CNeuron {

private:

  float           FValue;
  int             FHandle;
  string          FLabel;
  
  ENUM_INDICATOR  FIndicator;
  string          FIndicatorName;
  string          FSymbol;
  ENUM_TIMEFRAMES FTimeFrame;
  int             FBuffer;
  int             FOffset;

private:

  // Service procedures

  bool            IsLike (const string str, const string pattern);
  ENUM_INDICATOR  GetIndicatorType(string s);
//  ENUM_TIMEFRAMES GetTimeFrame(string s);
//  string          GetTimeFrameName(ENUM_TIMEFRAMES tf);
  ENUM_DATATYPE   GetDataType(string s);
  int             GetParamValue (string s);

  int             LoadValue(datetime aDateTime);
  
public:

  // Constructor procedures 
  
  void CNeuronIndicator ();
  virtual void ~CNeuronIndicator ();
  virtual bool Init (CXmlElement *aXmlElement, CNet *aNet);
//  virtual void Clear ();
  
  // Main procedures 

  virtual bool Run ();
  virtual float GetValue ();
};

//--------------------------------------------------------------------------------/
//                              CNeuronIndicator :: implementation                /
//--------------------------------------------------------------------------------/
  

  // Constructor procedures 

void CNeuronIndicator::CNeuronIndicator () 
{
  FHandle = INVALID_HANDLE;
};
  
void CNeuronIndicator::~CNeuronIndicator () 
{
//  IndicatorRelease(FHandle);
//  Clear();
};
  
bool CNeuronIndicator::Init (CXmlElement *aXmlElement, CNet *aNet) 
{
  bool r=true;
  MqlParam params[];
  
  r = CNeuron::Init(aXmlElement, aNet);
    
  FIndicatorName=FXmlElement.GetChildText("INDICATOR");
    
  FIndicator=GetIndicatorType(FIndicatorName);
    
  if (FIndicator == IND_CUSTOM) {
    ArrayResize(params,1);
    params[0].type = TYPE_STRING;
    params[0].string_value = FIndicatorName;
  };
    
  FSymbol = FXmlElement.GetChildText("SYMBOL");
    
  FTimeFrame = GetTimeFrame(FXmlElement.GetChildText("TIMEFRAME"));
    
  FBuffer=(int)StringToInteger(FXmlElement.GetChildText("BUFFER"));
    
  FOffset=(int)StringToInteger(FXmlElement.GetChildText("OFFSET"));

  FLabel = 
    FIndicatorName +
    " (" + FSymbol+
    ", " + FXmlElement.GetChildText ("TIMEFRAME") +
    ", Buf:" + FXmlElement.GetChildText ("BUFFER") +
    ", Off:" + FXmlElement.GetChildText ("OFFSET");
      
  for (int i=0; i<FXmlElement.GetChildCount(); i++)

    if (FXmlElement.GetChild(i).GetName()=="PARAM") 
    {
      FLabel += " ," + FXmlElement.GetChild(i).GetText();

      ArrayResize(params,ArraySize(params)+1);
        
      params[ArraySize(params)-1].type = GetDataType(FXmlElement.GetChild(i).GetAttributeValue("TYPE"));
       
      if (  // Double param
        (params[ArraySize(params)-1].type == TYPE_FLOAT) || 
        (params[ArraySize(params)-1].type == TYPE_DOUBLE)
      )            
        params[ArraySize(params)-1].double_value = StringToDouble(FXmlElement.GetChild(i).GetText());
      else 
      if (  // String param
        params[ArraySize(params)-1].type == TYPE_STRING
      )
        params[ArraySize(params)-1].string_value = FXmlElement.GetChild(i).GetText();
      else  // Integer param
        params[ArraySize(params)-1].integer_value = GetParamValue(FXmlElement.GetChild(i).GetText());
    };
      
  FLabel += ")";

  FHandle=INVALID_HANDLE;

  if(!SymbolInfoInteger(FSymbol,SYMBOL_SELECT)) 
  {
    if(GetLastError()==ERR_MARKET_UNKNOWN_SYMBOL) 
    {
      Print ("Error :   "+FSymbol);
      return(false);
    };
    SymbolSelect(FSymbol,true);
  };

  FHandle = IndicatorCreate (FSymbol,FTimeFrame,FIndicator,ArraySize(params),params);
      
  return(r && FHandle!=INVALID_HANDLE);
};
  
/*void CNeuronIndicator::Clear () 
{
  IndicatorRelease(FHandle);
  FHandle = INVALID_HANDLE;
};*/
  
  // Main procedures 

bool CNeuronIndicator::Run () 
{
  int i =0;
  int r=LoadValue(FNet.FPoint);
  while (r>0) {
    Sleep(100);
    r=LoadValue(FNet.FPoint);
    ++i;
    if (i>100)
    {
      i = 0;
      Print ("Neuron ", FXmlElement.GetAttributeValue("NAME"), " timeout 10 sec, at ", FNet.FPoint, " load err: ", r);
    }
  };

  return r==0;
};
  
float CNeuronIndicator::GetValue () 
{
  return (FValue);
};

  // Service procedures
  
/*bool CNeuronIndicator::IsLike (const string str, const string pattern) 
{
  return (
    str == pattern || 
    (
      StringSubstr (pattern,StringLen(pattern)-1,1) == "*" &&
      StringSubstr (str,0,StringLen(pattern)-1) == StringSubstr(pattern,0,StringLen(pattern)-1)
    )
  );
}; */

ENUM_INDICATOR CNeuronIndicator::GetIndicatorType(string s) 
{
  if (s=="AC") return(IND_AC);
  if (s=="AD") return(IND_AD);
  if (s=="ADX") return(IND_ADX);
  if (s=="ADXW") return(IND_ADXW);
  if (s=="ALLIGATOR") return(IND_ALLIGATOR);
  if (s=="AMA") return(IND_AMA);
  if (s=="AO") return(IND_AO);
  if (s=="ATR") return(IND_ATR);
  if (s=="BANDS") return(IND_BANDS);
  if (s=="BEARS") return(IND_BEARS);
  if (s=="BULLS") return(IND_BULLS);
  if (s=="BWMFI") return(IND_BWMFI);
  if (s=="CCI") return(IND_CCI);
  if (s=="CHAIKIN") return(IND_CHAIKIN);
  if (s=="DEMA") return(IND_DEMA);
  if (s=="DEMARKER") return(IND_DEMARKER);
  if (s=="ENVELOPES") return(IND_ENVELOPES);
  if (s=="FORCE") return(IND_FORCE);
  if (s=="FRACTALS") return(IND_FRACTALS);
  if (s=="FRAMA") return(IND_FRAMA);
  if (s=="GATOR") return(IND_GATOR);
  if (s=="ICHIMOKU") return(IND_ICHIMOKU);
  if (s=="MA") return(IND_MA);
  if (s=="MACD") return(IND_MACD);
  if (s=="MFI") return(IND_MFI);
  if (s=="MOMENTUM") return(IND_MOMENTUM);
  if (s=="OBV") return(IND_OBV);
  if (s=="OSMA") return(IND_OSMA);
  if (s=="RSI") return(IND_RSI);
  if (s=="RVI") return(IND_RVI);
  if (s=="SAR") return(IND_SAR);
  if (s=="STDDEV") return(IND_STDDEV);
  if (s=="STOCHASTIC") return(IND_STOCHASTIC);
  if (s=="TEMA") return(IND_TEMA);
  if (s=="TRIX") return(IND_TRIX);
  if (s=="VIDYA") return(IND_VIDYA);
  if (s=="VOLUMES") return(IND_VOLUMES);
  if (s=="WPR") return(IND_WPR);
  return(IND_CUSTOM);
}

ENUM_DATATYPE CNeuronIndicator::GetDataType(string s) 
{
  if (s=="BOOL") return(TYPE_BOOL);
  if (s=="CHAR") return(TYPE_CHAR);
  if (s=="UCHAR") return(TYPE_UCHAR);
  if (s=="SHORT") return(TYPE_SHORT);
  if (s=="USHORT") return(TYPE_USHORT);
  if (s=="COLOR") return(TYPE_COLOR);
  if (s=="INT") return(TYPE_INT);
  if (s=="UINT") return(TYPE_UINT);
  if (s=="DATATIME") return(TYPE_DATETIME);
  if (s=="LONG") return(TYPE_LONG);
  if (s=="ULONG") return(TYPE_ULONG);
  if (s=="FLOAT") return(TYPE_FLOAT);
  if (s=="DOUBLE") return(TYPE_DOUBLE);
  if (s=="STRING") return(TYPE_STRING);
  return(TYPE_INT);
}

int CNeuronIndicator::GetParamValue (string s) 
{
  if (s=="PRICE_CLOSE") return(PRICE_CLOSE);
  if (s=="PRICE_OPEN") return(PRICE_OPEN);
  if (s=="PRICE_HIGH") return(PRICE_HIGH);
  if (s=="PRICE_LOW") return(PRICE_LOW);
  if (s=="PRICE_MEDIAN") return(PRICE_MEDIAN);
  if (s=="PRICE_TYPICAL") return(PRICE_TYPICAL);
  if (s=="PRICE_WEIGHTED") return(PRICE_WEIGHTED);
  
  if (s=="VOLUME_TICK") return(VOLUME_TICK);
  if (s=="VOLUME_REAL") return(VOLUME_REAL);

  if (s=="STO_LOWHIGH") return(STO_LOWHIGH);
  if (s=="STO_CLOSECLOSE") return(STO_CLOSECLOSE);
  
  if (s=="MODE_SMA") return(MODE_SMA);
  if (s=="MODE_EMA") return(MODE_EMA);
  if (s=="MODE_SMMA") return(MODE_SMMA);
  if (s=="MODE_LWMA") return(MODE_LWMA);
  return((int)StringToInteger(s));
};

int CNeuronIndicator::LoadValue (datetime aDateTime) 
{
  //  :
  //    0 -       FValue
  //   >0 -  
  //   <0 -      
  
  FValue=FLT_MAX;

  if (FHandle==INVALID_HANDLE) 
    return(-1);  //  

  datetime times[100];
  datetime series_firstdate, terminal_firstdate, server_firstdate;

  if (CopyTime(FSymbol,FTimeFrame,aDateTime,1,times)==1) 
  { 
    if (aDateTime<times[0] || aDateTime>=(times[0]+PeriodSeconds(FTimeFrame))) 
      return(-2);  //       aDateTime
    
    if (BarsCalculated(FHandle)<Bars(FSymbol,FTimeFrame)) 
      return(1);   // ,   
      
    double v[];
    if (CopyBuffer(FHandle,FBuffer,times[0],FOffset+1,v)!=(FOffset+1)) 
      return(-3);  // -   , ,   

    if (v[0]!=EMPTY_VALUE) 
    {
      FValue = (float) v[0];
      return(0);
    }
    else
      FValue = FLT_MAX;
    ;
  };  

  if(MQL5InfoInteger(MQL5_PROGRAM_TYPE)==PROGRAM_INDICATOR && Period()==FTimeFrame && Symbol()==FSymbol)
    return(-4);   //      
  
  if (!SeriesInfoInteger(FSymbol,FTimeFrame,SERIES_SYNCRONIZED))
    return(1);    //  

  int bars=Bars(FSymbol,FTimeFrame);
  
  if (bars>=TerminalInfoInteger(TERMINAL_MAXBARS)) 
    return(-5);

  if (!SeriesInfoInteger(FSymbol,FTimeFrame,SERIES_FIRSTDATE,series_firstdate)) 
    return(1);  

  if (!SeriesInfoInteger(FSymbol,FTimeFrame,SERIES_TERMINAL_FIRSTDATE,terminal_firstdate)) 
    return(1);  

  if (aDateTime<series_firstdate) 
  {
    CopyTime(FSymbol,FTimeFrame,aDateTime,100,times);    
    return(1);
  };

    if (series_firstdate>terminal_firstdate) {
      CopyTime(FSymbol,FTimeFrame,terminal_firstdate,1,times);    
      return(1);
    }

    if (!SeriesInfoInteger(FSymbol,PERIOD_M1,SERIES_SERVER_FIRSTDATE,server_firstdate))
      return(1);

    if (series_firstdate<=series_firstdate)
      return(-6); 

    CopyTime(FSymbol,FTimeFrame,terminal_firstdate,100,times);    
    return(1);
  };



