//-----------------------------------------------------------------------------
//                                  CNeuronLinear                             !
//-----------------------------------------------------------------------------

class CNeuronLinear : public CNeuronMultiLink {

protected:

  float   FSum;

public:

  // Constructor procedures 
  
  void CNeuronLinear ();

  // Main procedures 

  virtual bool Run ();
  virtual float GetValue ();
  virtual float GetdFdLink (const CLink *aLink);
  virtual float GetdFdNeuron (const CNeuron *aNeuron);
  
  // Service procedures
  
  float   GetLeftWeight (int idx);
  CNeuron* GetLeftNeuron (int idx);
};

//--------------------------------------------------------------------------------/
//                              CNeuronLinear :: implementation                   /
//--------------------------------------------------------------------------------/
  
void CNeuronLinear::CNeuronLinear () 
{
  FSum=FLT_MAX;
};
  
float CNeuronLinear::GetValue () 
{
  return (FSum);
};

bool CNeuronLinear::Run () 
{
  FSum=FBiasWeight;
  for (int i=0; i<ArraySize(FLeftLinks); ++i) 
    FSum+=FLeftLinks[i].GetLeftNeuron().GetValue() * FLeftLinks[i].GetWeight();
  return FSum != FLT_MAX;
};

float CNeuronLinear::GetdFdLink (const CLink *aLink) 
{
  if (aLink.GetRightNeuron()==GetPointer(this)) 
  {
    return (aLink.GetLeftNeuron().GetValue());
  } 
  else 
  {
    float dFdLink = 0;
    for (int i=0; i<ArraySize(FLeftLinks); ++i)
      dFdLink += GetLeftWeight(i) * FLeftLinks[i].GetLeftNeuron().GetdFdLink(aLink);
    return (dFdLink);
  };
};
  
float CNeuronLinear::GetdFdNeuron (const CNeuron *aNeuron) 
{
  if (aNeuron==GetPointer(this))
    return(1.0f);
  else 
  {
    float dFdNeuron = 0.0f;
    for (int i=0; i<ArraySize(FLeftLinks); ++i )
      dFdNeuron += GetLeftWeight(i) * FLeftLinks[i].GetLeftNeuron().GetdFdNeuron(aNeuron);
    return(dFdNeuron);
  };
};

float CNeuronLinear::GetLeftWeight (int idx)
{
  return ( ((CLinkWeight*)FLeftLinks[idx]).GetWeight() );
};

CNeuron* CNeuronLinear::GetLeftNeuron (int idx)
{
  return ( FLeftLinks[idx].GetLeftNeuron() );
};
