//-----------------------------------------------------------------------------
//                                  CNeuronMSE                                !
//-----------------------------------------------------------------------------

class CNeuronMSE : public CNeuron 
{
protected :

  float FDiff;
  CLink FCalcNeuronLink;
  CLink FRealNeuronLink;

public :

  // Constructor procedures 

  void          CNeuronMSE ();
  virtual void ~CNeuronMSE ();
  
  virtual bool Init (CXmlElement *aXmlElement, CNet *aNet);
  virtual void Upload ();
//  virtual void Clear ();
  
  // Main procedures 

  virtual bool Run ();
  int GetLayer ();  
  virtual float GetValue ();
  virtual float GetdFdLink (const CLink *aLink);
  virtual float GetdFdNeuron (const CNeuron *aNeuron);
};

//--------------------------------------------------------------------------------/
//                              CNeuronMSE :: implementation                      /
//--------------------------------------------------------------------------------/
  
  // Constructor procedures 

void CNeuronMSE::CNeuronMSE ()
{
  FDiff = FLT_MAX;
  FCalcNeuronLink.SetLeftNeuron (NULL);
  FCalcNeuronLink.SetRightNeuron (GetPointer(this));
  FRealNeuronLink.SetLeftNeuron (NULL);
  FRealNeuronLink.SetRightNeuron (GetPointer(this));
};

void CNeuronMSE::~CNeuronMSE ()
{
};

bool CNeuronMSE::Init (CXmlElement *aXmlElement, CNet *aNet)
{
  bool r = CNeuron::Init(aXmlElement, aNet);
  FCalcNeuronLink.SetLeftNeuron (FNet.GetNeuron (FXmlElement.GetAttributeValue("CALC")));
  FRealNeuronLink.SetLeftNeuron (FNet.GetNeuron (FXmlElement.GetAttributeValue("REAL")));
  return r && (FCalcNeuronLink.GetLeftNeuron()!=NULL) && (FRealNeuronLink.GetLeftNeuron()!=NULL);
};

void CNeuronMSE::Upload () {
  CNeuron::Upload();
  FXmlElement.GetAttribute("CALC").SetValue(FCalcNeuronLink.GetLeftNeuron().GetXmlElement().GetAttributeValue("NAME"));
  FXmlElement.GetAttribute("REAL").SetValue(FRealNeuronLink.GetLeftNeuron().GetXmlElement().GetAttributeValue("NAME"));
};

  // Main procedures 

bool CNeuronMSE::Run ()
{
  FDiff = FCalcNeuronLink.GetLeftNeuron().GetValue() - FRealNeuronLink.GetLeftNeuron().GetValue();
  return FDiff != FLT_MAX;
};

int CNeuronMSE::GetLayer ()
{
  return MathMax(FCalcNeuronLink.GetLeftNeuron().GetLayer(), FRealNeuronLink.GetLeftNeuron().GetLayer()) + 1;
};

float CNeuronMSE::GetValue ()
{
  return 1/2*FDiff*FDiff;
};

float CNeuronMSE::GetdFdLink (const CLink *aLink)
{
  return FDiff * (FCalcNeuronLink.GetLeftNeuron().GetdFdLink(aLink) - FRealNeuronLink.GetLeftNeuron().GetdFdLink(aLink));;
};

float CNeuronMSE::GetdFdNeuron (const CNeuron *aNeuron)
{
 if (aNeuron == GetPointer(this))
   return 1.0f;
 else
   return FDiff * (FCalcNeuronLink.GetLeftNeuron().GetdFdNeuron(aNeuron) - FRealNeuronLink.GetLeftNeuron().GetdFdNeuron(aNeuron));
};