//-----------------------------------------------------------------------------
//                                  CNeuronMultiLink                          !
//-----------------------------------------------------------------------------

class CNeuronMultiLink : public CNeuron {

protected:

  float FBiasWeight;
  CLinkWeight   *FLeftLinks[];
  void UnTemplateLinks (CXmlElement &xmlItem); 
  bool LoadLinks (CXmlElement &xmlItem); 

public:
  
  void CNeuronMultiLink ();
  virtual void ~CNeuronMultiLink ();
  virtual bool Init (CXmlElement *aXmlElement, CNet *aNet);
  virtual void Upload ();
  
  virtual int GetLayer ();

  int    GetLeftLinkCount ();
  CLink* GetLeftLink (int idx);
  int    AddLeftLink (CLink* aLink);
  void   DelLeftLink (int idx);
  
};

//--------------------------------------------------------------------------------/
//                              CNeuronMultiLink :: implementation                   /
//--------------------------------------------------------------------------------/
  
void CNeuronMultiLink::CNeuronMultiLink ()
{
  ArrayResize(FLeftLinks,0);
};

void CNeuronMultiLink::~CNeuronMultiLink ()
{
  for (int i=ArraySize(FLeftLinks)-1; i>=0; --i)
    DelLeftLink(i);
};

bool CNeuronMultiLink::Init (CXmlElement *aXmlElement, CNet *aNet)
{
  bool r = CNeuron::Init (aXmlElement, aNet);
  
 // UnTemplateLinks (FXmlElement);
  
  FBiasWeight = (float) StringToDouble(aXmlElement.GetAttributeValue("BIAS"));
  r = r && LoadLinks (FXmlElement);
    
  return r;
};

void CNeuronMultiLink::Upload ()
{
  CNeuron::Upload ();
  for (int i=0; i<ArraySize(FLeftLinks); ++i)
    FLeftLinks[i].Upload ();
};

int CNeuronMultiLink::GetLayer ()
{
  int max_layer=0;
  for (int i=0; i<ArraySize(FLeftLinks); ++i) {
    int layer = FLeftLinks[i].GetLeftNeuron().GetLayer();
    if (layer > max_layer) 
      max_layer = layer;
  };
  return max_layer + 1;
};

int CNeuronMultiLink::GetLeftLinkCount() 
{
  return (ArraySize(FLeftLinks));
};

CLink* CNeuronMultiLink::GetLeftLink(int idx) 
{
  return (FLeftLinks[idx]);
};

int CNeuronMultiLink::AddLeftLink(CLink* aLink) 
{
  ArrayResize(FLeftLinks,ArraySize(FLeftLinks)+1);
  FLeftLinks[ArraySize(FLeftLinks)-1]=aLink;
  return (ArraySize(FLeftLinks)-1);
};
  
void CNeuronMultiLink::DelLeftLink(int idx) 
{
  FLeftLinks[idx].GetLeftNeuron().DelRightLink(GetPointer(FLeftLinks[idx]));
  delete FLeftLinks[idx];
  for (int i=idx+1; i<ArraySize(FLeftLinks); ++i)
    FLeftLinks[i-1]=FLeftLinks[i];
  ArrayResize(FLeftLinks, ArraySize(FLeftLinks)-1);
};  

  // Protected procedures

void CNeuronMultiLink::UnTemplateLinks (CXmlElement &xmlItem) 
{
  int i=0;
  while (i<xmlItem.GetChildCount()) {
    if (xmlItem.GetChild(i).GetName() == "LINK")
    {
      CXmlElement* xmlLink = xmlItem.GetChild(i);
      for (int j=0; j<ArraySize(FNet.FNeurons); j++) 
        if (IsLike(FNet.FNeurons[j].GetXmlElement().GetAttributeValue("NAME"),xmlLink.GetAttributeValue("FROM"))) 
        {
          CXmlElement* new_link = new CXmlElement;
          xmlLink.CopyTo (new_link);
          new_link.GetAttribute("FROM").SetValue (FNet.FNeurons[j].GetXmlElement().GetAttributeValue("NAME"));
          xmlItem.ChildInsertAt(new_link, i);
          ++i;
        };
      xmlItem.ChildDelete (xmlLink);
      --i;
    } 
    else
      UnTemplateLinks (xmlItem.GetChild(i));
    ++i;
  };
};
  
bool CNeuronMultiLink::LoadLinks (CXmlElement &xmlItem) 
{
  bool r=true;
  
  for (int i=0; i<xmlItem.GetChildCount(); i++) 
    if (xmlItem.GetChild(i).GetName() == "LINK")
    {
      ArrayResize(FLeftLinks,ArraySize(FLeftLinks)+1);
      FLeftLinks[ArraySize(FLeftLinks)-1] = CreateLink ("WEIGHT");
        
      r = r && FLeftLinks[ArraySize(FLeftLinks)-1].Init(xmlItem.GetChild(i),GetPointer(this));
        
      for (int j = 0; j < ArraySize(FLeftLinks)-1; ++j)
        r = r && (
           FLeftLinks[j].GetXmlElement().GetAttributeValue("TO") != 
           FLeftLinks[ArraySize(FLeftLinks)-1].GetXmlElement().GetAttributeValue("TO")
        );
    } 
    else
      r = r && LoadLinks (xmlItem.GetChild(i));

  return r;
};
