//-----------------------------------------------------------------------------
//                                  CNeuronProfit                             !
//-----------------------------------------------------------------------------

class CNeuronProfit : public CNeuron 
{
protected :

  float FSpread;			      	// Spread   
  float FValue;

  CNeuron* FInLotNeuron;
  CNeuron* FOutLotNeuron;
  CNeuron* FBarCostNeuron;		//         ( >0  , <0  

  float dFdOut ();

public :

  // Constructor procedures 

  void          CNeuronProfit ();
  virtual void ~CNeuronProfit ();
  
  virtual bool Init (CXmlElement *aXmlElement, CNet *aNet);
  virtual void Upload ();
//  virtual void Clear ();
  
  // Main procedures 

  virtual bool Run ();
  int GetLayer ();  
  virtual float GetValue ();
  virtual float GetdFdLink (const CLink *aLink);
  virtual float GetdFdNeuron (const CNeuron *aNeuron);
};

//--------------------------------------------------------------------------------/
//                              CNeuronMSE :: implementation                      /
//--------------------------------------------------------------------------------/
  
  // Constructor procedures 

void CNeuronProfit::CNeuronProfit ()
{
  FValue = FLT_MAX;
  FSpread = FLT_MAX;
  FInLotNeuron = NULL;
  FOutLotNeuron = NULL;
  FBarCostNeuron = NULL;
};

void CNeuronProfit::~CNeuronProfit ()
{
};

bool CNeuronProfit::Init (CXmlElement *aXmlElement, CNet *aNet)
{
  bool r = CNeuron::Init(aXmlElement, aNet);
 
  FInLotNeuron = FNet.GetNeuron (FXmlElement.GetAttributeValue("INLOT"));
  FOutLotNeuron = FNet.GetNeuron (FXmlElement.GetAttributeValue("OUTLOT"));
  FBarCostNeuron = FNet.GetNeuron (FXmlElement.GetAttributeValue("BARCOST"));
  FSpread = (float)StringToDouble(FXmlElement.GetAttributeValue("SPREAD"));
  return r && (FInLotNeuron!=NULL) && (FOutLotNeuron!=NULL) && (FBarCostNeuron!=NULL);
};

void CNeuronProfit::Upload () {
  CNeuron::Upload();
  FXmlElement.GetAttribute("INLOT").SetValue(FInLotNeuron.GetXmlElement().GetAttributeValue("NAME"));
  FXmlElement.GetAttribute("OUTLOT").SetValue(FInLotNeuron.GetXmlElement().GetAttributeValue("NAME"));
  FXmlElement.GetAttribute("BARCOST").SetValue(FInLotNeuron.GetXmlElement().GetAttributeValue("NAME"));
  FXmlElement.GetAttribute("SPREAD").SetValue(DoubleToString (FSpread,-8));
};

  // Main procedures 

bool CNeuronProfit::Run ()
{
	float in_lot = FInLotNeuron.GetValue();
	float out_lot = FOutLotNeuron.GetValue();
	float barcost = FBarCostNeuron.GetValue();

	float spread_lot=																			
		(in_lot>0)																		
			? ((out_lot<0) ? (-out_lot) : ((out_lot>in_lot) ? (out_lot-in_lot) : (0)))	
			: ((out_lot>0) ? ( out_lot) : ((out_lot<in_lot) ? (in_lot-out_lot) : (0)));

  FValue = out_lot * barcost - FSpread * spread_lot;
  return FValue != FLT_MAX;
};

int CNeuronProfit::GetLayer ()
{
  return 
	  MathMax (
		  MathMax (
		   FInLotNeuron.GetLayer(),
		   FOutLotNeuron.GetLayer()
		  ),  
	  	FBarCostNeuron.GetLayer()
	  )
	  + 1;
};

float CNeuronProfit::GetValue ()
{
    return FValue;
};

float CNeuronProfit::dFdOut ()
{
	float in_lot = FInLotNeuron.GetValue();
	float out_lot = FOutLotNeuron.GetValue();
	float barcost = FBarCostNeuron.GetValue();

	float dSpreadLot_dOut =												
		(in_lot>0)														
			? ((out_lot<0) ? (-1.0f) : ((out_lot>in_lot) ? ( 1.0f) : (0)))	
			: ((out_lot>0) ? ( 1.0f) : ((out_lot<in_lot) ? (-1.0f) : (0)));	

	return barcost - FSpread * dSpreadLot_dOut;
};

float CNeuronProfit::GetdFdLink (const CLink *aLink)
{
    return dFdOut() * FOutLotNeuron.GetdFdLink(aLink);
};

float CNeuronProfit::GetdFdNeuron (const CNeuron *aNeuron)
{
 if (aNeuron == GetPointer(this))
   return 1.0f;
 else
    return dFdOut() * FOutLotNeuron.GetdFdNeuron(aNeuron);
};